<#  
.SYNOPSIS  
	Sets the WindowsEmailAddress field to first.last@smtpdomain.com
	
.DESCRIPTION  
	This script is for NON EXCHANGE ENVIRONMENTS ONLY. It is designed to set the WindowsEmailAddress field of AD user accounts. It targets a specific OU and all child OUs.
	Note: It removes spaces in names to adhere to RFC 2821
	
.NOTES  
    Version      	   		: 1.1 - See changelog at http://www.ehloworld.com/605
		Wish list						: better error trapping
    Rights Required			: Rights to update user accounts
    Sched Task Req'd		: No
    Exchange Version		: N/A
    Lync Version				: N/A
    Author       				: Pat Richard, Exchange MVP
    Email/Blog/Twitter	: pat@innervation.com 	http://www.ehloworld.com @patrichard
    Dedicated Post			: http://www.ehloworld.com/240
    Disclaimer   				: You running this script means you won't blame me if this breaks your stuff.
    Info Stolen from 		: (Get-Culture)	http://social.technet.microsoft.com/Forums/en/ITCG/thread/ab069b93-cf99-43a2-8226-014ceb0f05c5
    										: (OU picker) http://thepip3r.blogspot.com/2011/06/powershell-guis-active-directory.html
    										
.LINK  
	http://www.ehloworld.com/240
	
.EXAMPLE
	.\Set-WindowsEmailAddress.ps1 -SMTPDomain "contoso.com"
	
.INPUTS
	None. You cannot pipe objects to this script.
#>
#Requires -Version 2.0
param(
	[parameter(ValueFromPipeline=$false, ValueFromPipelineByPropertyName=$false, Mandatory=$false)] 
	[string] $FilenameTranscript = $MyInvocation.MyCommand.Name + " " + (hostname)+ " {0:yyyy-MM-dd hh-mmtt}.log" -f (Get-Date),
	[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$true)] 
	[string] $SMTPDomainName
)

Start-Transcript -path .\$FilenameTranscript | Out-Null

function Get-ModuleStatus { 
	param	(
		[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$true, HelpMessage="No module name specified!")] 
		[string]$name
	)
	if(!(Get-Module -name "$name")) { 
		if(Get-Module -ListAvailable | ? {$_.name -eq "$name"}) { 
			Import-Module -Name "$name" 
			# module was imported
			return $true
		} else {
			# module was not available
			return $false
		}
	}else {
		# module was already imported
		# Write-Host "$name module already imported"
		return $true
	}
} # end function Get-ModuleStatus

function Remove-ScriptVariables($path) {  
	$result = Get-Content $path |  
	ForEach { if ( $_ -match '(\$.*?)\s*=') {      
			$matches[1]  | ? { $_ -notlike '*.*' -and $_ -notmatch 'result' -and $_ -notmatch 'env:'}  
		}  
	}  
	ForEach ($v in ($result | Sort-Object | Get-Unique)){		
		# Write-Host "Removing" $v.replace("$","")
		Remove-Variable ($v.replace("$","")) -ErrorAction SilentlyContinue
	}
} # end function Get-ScriptVariables

function Add-Node {
	param (
		$selectedNode,
		$name
	)
	$newNode = new-object System.Windows.Forms.TreeNode
 	$newNode.Name = $name
	$newNode.Text = $name
	$selectedNode.Nodes.Add($newNode) | Out-Null
	return $newNode  
} # end function Add-Node

function Get-NextLevel {
	param (
		$selectedNode,
		$dn
	)

	$OUs = Get-ADObject -Filter 'ObjectClass -eq "organizationalUnit" -or ObjectClass -eq "container"' -SearchScope OneLevel -SearchBase $dn
	
	If ($OUs -eq $null) {
		$node = Add-Node $selectedNode $dn
 	} Else {
		$node = Add-Node $selectedNode $dn

		$OUs | % {
   		Get-NextLevel $node $_.distinguishedName
		}
 	} 
} # end function Get-NextLevel

function Build-TreeView {
	if ($treeNodes)  {
  	$treeview1.Nodes.remove($treeNodes)
		$form1.Refresh()
  }

	$treeNodes = New-Object System.Windows.Forms.TreeNode
	$treeNodes.text = "Active Directory Hierarchy"
	$treeNodes.Name = "Active Directory Hierarchy"
	$treeNodes.Tag = "root"
	$treeView1.Nodes.Add($treeNodes) | Out-Null

	$treeView1.add_AfterSelect({
	$textbox1.Text = $this.SelectedNode.Name
	})
	
	#Generate Module nodes
  $OUs = Get-NextLevel $treeNodes $strDomainDN

	$treeNodes.Expand()  
} # end function Build-TreeView

#Generated Form Function  
function GenerateForm {
	#region Import the Assemblies
  # Import-Module ActiveDirectory
	[reflection.assembly]::loadwithpartialname("System.Windows.Forms") | Out-Null
  [reflection.assembly]::loadwithpartialname("System.Drawing") | Out-Null
  $objIPProperties = [System.Net.NetworkInformation.IPGlobalProperties]::GetIPGlobalProperties()
	$strDNSDomain = $objIPProperties.DomainName.toLower()
	$strDomainDN = $strDNSDomain.toString().split('.'); foreach ($strVal in $strDomainDN) {$strTemp += "dc=$strVal,"}; $strDomainDN = $strTemp.TrimEnd(",").toLower()
	#endregion
  
  #region Generated Form Objects
  $form1 = New-Object System.Windows.Forms.Form
  $treeView1 = New-Object System.Windows.Forms.TreeView
  $label1 = New-Object System.Windows.Forms.Label
 	$textbox1 = New-Object System.Windows.Forms.TextBox
 	$InitialFormWindowState = New-Object System.Windows.Forms.FormWindowState
  #endregion Generated Form Objects
  
  #----------------------------------------------
  #Generated Event Script Blocks
  #----------------------------------------------
  $button1_OnClick=
  {
  $form1.Close()
  
  }
  
  $OnLoadForm_StateCorrection=
  {Build-TreeView
  }
	
	#----------------------------------------------
  #region Generated Form Code
  $form1.Text = "Select source OU"
 	$form1.Name = "form1"
 	$form1.DataBindings.DefaultDataSourceUpdateMode = 0
  $form1.ClientSize = New-Object System.Drawing.Size(400,500)
  
  $treeView1.Size = New-Object System.Drawing.Size(350,375)
 	$treeView1.Name = "treeView1"
 	$treeView1.Location = New-Object System.Drawing.Size(15,15)
 	$treeView1.DataBindings.DefaultDataSourceUpdateMode = 0
  $treeView1.TabIndex = 0
  $form1.Controls.Add($treeView1)
 
 	$label1.Name = "label1"
 	$label1.Location = New-Object System.Drawing.Size(15,400)
 	$label1.Size = New-Object System.Drawing.Size(350,20)
 	$label1.Text = "Selected OU:"
	$form1.Controls.Add($label1)
  
  $textbox1.Name = "textbox1"
 	$textbox1.Location = New-Object System.Drawing.Size(15,425)
 	$textbox1.Size = New-Object System.Drawing.Size(350,20)
 	$textbox1.Text = ""
	$form1.Controls.Add($textbox1)

  $OKButton = New-Object System.Windows.Forms.Button
	$OKButton.Location = New-Object System.Drawing.Size(290,460)
	$OKButton.Size = New-Object System.Drawing.Size(75,23)
	$OKButton.Text = "OK"
	$OKButton.Add_Click({$global:ou=$textbox1.text;$form1.Close()})
	$form1.Controls.Add($OKButton)
	
	#endregion Generated Form Code
  
  #Save the initial state of the form
  $InitialFormWindowState = $form1.WindowState
  #Init the OnLoad event to correct the initial state of the form
  $form1.add_Load($OnLoadForm_StateCorrection)
  #Show the Form
  $form1.ShowDialog()| Out-Null

} # end function GenerateForm

if ((Get-ModuleStatus ActiveDirectory) -eq $false){
	$error.clear()
	Write-Host "Installing the Active Directory module..." -ForegroundColor yellow
	Get-ModuleStatus ServerManager
	Add-WindowsFeature RSAT-AD-PowerShell
	if ($error){
		Write-Host "Active Directory module could not be installed. Exiting..." -ForegroundColor red; 
		if ($transcript){Stop-Transcript}
		exit
	}
}

GenerateForm
if ($ou){
	[object] $objUsers = Get-AdUser -filter * -ResultSetSize $null | ? {$_.DistinguishedName -match $ou -and $_.GivenName -ne $null -and $_.Surname -ne $null}
	if ($objUsers -ne $null){
		foreach ($objUser in $objUsers){
			[string] $strEmail = ((Get-Culture).TextInfo.ToTitleCase($objUser.GivenName.ToLower()).replace(" ","")) + "." + ((Get-Culture).TextInfo.ToTitleCase($objUser.Surname.ToLower()).replace(" ","")) + "@" + $SMTPDomainName
			# [string] $strEmail = ($objUser.GivenName.ToLower().replace(" ","")) + "." + ($objUser.Surname.ToLower().replace(" ","")) + "@" + $SMTPDomainName
			[string] $strSam = $objuser.samaccountname
			write-host $objUser.samaccountname -nonewline
			write-host " ($strEmail)" -foregroundcolor green
			Set-AdUser $strSam -emailaddress $strEmail
		}
	}else{
		Write-Host "No users found" -ForegroundColor yellow
	}

	Write-Host "`nDone" -foregroundcolor green
}

Stop-Transcript | Out-Null

# let's clean up after ourselves per http://www.ucblogs.net/blogs/exchange/archive/2011/07/08/Cleaning-up-script-variables-in-PowerShell.aspx
Remove-ScriptVariables($MyInvocation.MyCommand.Name)